<?php
/**
 * Common library of functions used by UOC Services.
 *
 * @package uocKpax
 */


 /**
 * Tests if the system admin has enabled UOC
 *
 * @param void
 * @return bool
 */
function uocKpax_allow_sign_on_with_uoc() {
	if (!$consumer_clientId = elgg_get_plugin_setting('consumer_clientId', 'uocKpax')) {
		 error_log('consumer_clientId');
		return FALSE;
	}
	
	if (!$consumer_key = elgg_get_plugin_setting('consumer_secret', 'uocKpax')) {
		 error_log('oscar consumer_key');
		return FALSE;
	}

	if (!$consumer_host = elgg_get_plugin_setting('consumer_host', 'uocKpax')) {
		 error_log(consumer_host);
		return FALSE;
	}
	
	return elgg_get_plugin_setting('sign_on', 'uocKpax') == 'yes';
}


/**
 * Obte les dades del usuari de la UOC. Es crida aquest metode com a resposta a /oauth/authorize
 * per tant, aqui ja disposem del code com a paremetre en la url
 */
function uocKpax_login() {

	error_log('oscar uocKpax_login');

	if(elgg_is_logged_in()) { 
		forward();
	}
	
	//acces to UOC API Service  
	$uoc = uocKpaxservice_api();
	
	//Obtenim les dades de l'usuari UOC.
	//Aixo ja ho fa tot, aixi queda encapsulat dins la api UOC -> obte el token a partir del code i les dades del usuari UOC en format JSON
	$userUOC = $uoc->getUser(elgg_get_site_url() .'uocKpax/login');
    
	//Test
	error_log('oscar uocKpax_login: userUOC.email = ' .$userUOC->email); //aqui si mires la traça veuras que hemm obtingut les dades de la UOC
	error_log('oscar uocKpax_login: userUOC.username = ' .$userUOC->username); 
	error_log('oscar uocKpax_login: userUOC.name = ' .$userUOC->name); 
	//end test
	
	//Comprovar si usuari ja esta donat d'alta a elgg (es comprova per email)
	$users = get_user_by_email($userUOC->email); // get_user_by_username($userUOC->username);
	
	if (count($users) >= 1) {  //usuari ja existeix
		error_log('oscar uocKpax_login: usuario existe -> login');
		//fem login
		uocKpax_loginElgg($users[0]);
	} else {
		error_log('oscar uocKpax_login: usuario no existe');
	    //Crear usuari si no existeix
	    $newElggUser = uocKpax_createNewUser($userUOC); //crear usuari a Elgg. TODO: crear a KPAX tambe
	    // Login a Elgg
	    uocKpax_loginElgg($newElggUser);
	}
	

	forward();
}


/** 
 * Crea un nou usuari a Elgg
 * @param $uocUser JSON amb les dades del usuari UOC
 * @return $newUser nou usuari Elgg
 */
function uocKpax_createNewUser($uocUser) {
	//les dades necessaries per crear un nou usuari son: username, name, email i password
	
	//obtenim les dades necessaries del usuario UOC per crear usuari a elgg
	
	$password='uocTest1234'; //Per test. -> TODO: generar pwd aleatori  -> generate_random_cleartext_password();
	
	$newUser = new ElggUser();
	$newUser->username = 'uoc.edu_' . $uocUser->username; //Aixo per seguir el que es fa quan es crida el KpaxSrv. Sembla que s'espera que un usuari uoc a elgg sigui del tipus uoc.edu_elquesigui
	$newUser->name = $uocUser->name;
	$newUser->email = $uocUser->email;
	$newUser->access_id = ACCESS_PUBLIC;
	$newUser->password = generate_user_password($newUser, $password); 
	$newUser->owner_guid = 0;
	$newUser->container_guid = 0;
	
	//guardem el nou usuari a elgg
	//TODO fer el mateix a KPAX
	if (!$newUser->save()) {
		error_log('oscar uocKpax_createNewUser->KO');
		register_error(elgg_echo('uocKpax_createNewUser->error'));
		forward();
	} else {
	    error_log('oscar uocKpax_createNewUser->OK');
	}
	
	// Guardem un flag per saber que aquest usuari s'ha creat mitjançant proces UOC. TODO: si al final no es necessita es treu
	elgg_set_plugin_user_setting('uocUser', $newUser->username, $newUser->guid);

	return $newUser;
}

/** 
 * Inicia sessio a Elgg
 * @param $user usuari Elgg
 * @return $newUser nou usuari Elgg
 */
function uocKpax_loginElgg($user) {
	try {
		 login($user);
		 error_log('oscar uuocKpax_loginElgg: loginUser OK ');
		
		 forward();
		} catch (LoginException $e) {
			error_log('oscar uocKpax_loginElgg: loginUser KO');
			register_error($e->getMessage());
			forward(REFERER);
	    }	
}


/**
 * Returns the url to authorize the user.
 * @param string $next The return URL.
 */
function uocKpax_get_authorize_url($redirect_url='') {
    global $SESSION;
	
	if (!$redirect_url) {
		//default login page
		$redirect_url = elgg_get_site_url() .'uocKpax/login';
	}
	
	$uoc = uocKpaxservice_api();
	return $uoc->getLoginUrl($redirect_url);
}

/*
 * Access to API UOC
 * @param void
 */ 
function uocKpaxservice_api() {
	elgg_load_library('uoc');
	error_log('oscar secret:' . elgg_get_plugin_setting('consumer_secret', 'uocKpax'));
	return new UOC(array(
		'clientId' => elgg_get_plugin_setting('consumer_clientId', 'uocKpax'),
		'secret' => elgg_get_plugin_setting('consumer_secret', 'uocKpax'),
		'host' => elgg_get_plugin_setting('consumer_host', 'uocKpax')
	));	
}

?>